Package: rapsimng
Type: Package
Title: APSIM Next Generation
Version: 0.6.0
Authors@R: person("Bangyou", "Zheng", 
               email = "zheng.bangyou@gmail.com", 
               role = c("aut", "cre"))
Description: The Agricultural Production Systems sIMulator ('APSIM') is a widely
    used to simulate the agricultural systems for multiple crops. This package 
    is designed to create, modify and run 'apsimx' files in the 'APSIM' Next 
    Generation <https://www.apsim.info/>.
License: MIT + file LICENSE
URL: https://rapsimng.bangyou.me/, https://github.com/byzheng/rapsimng
BugReports: https://github.com/byzheng/rapsimng/issues
Encoding: UTF-8
Depends: R (>= 4.1.0)
Imports: jsonlite, tibble, magrittr, dplyr, purrr, rlang, DBI, RSQLite,
        tidyweather
Suggests: testthat, knitr, rmarkdown, tidyverse, methods
VignetteBuilder: knitr
Config/roxygen2/version: 8.1.0
NeedsCompilation: no
Packaged: 2026-08-24 06:09:38 UTC; zhe00a
Author: Bangyou Zheng [aut, cre]
Maintainer: Bangyou Zheng <zheng.bangyou@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-24 06:30:02 UTC
Built: R 4.5.2; ; 2026-08-24 09:43:24 UTC; unix
